/*
 *	@(#)HelloUniverse.java 1.16 02/04/01 15:03:54
 *
 * Copyright (c) 1996-2002 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistribution in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * Neither the name of Sun Microsystems, Inc. or the names of
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
 * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
 * EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN
 * OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR
 * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE SOFTWARE,
 * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * You acknowledge that Software is not designed,licensed or intended
 * for use in the design, construction, operation or maintenance of
 * any nuclear facility.
 */

import java.io.*;
import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class CellSimClient extends Applet {

	public final Color3f black = new Color3f(0f,0f,0f);
	public final Color3f white = new Color3f(0f,0f,0f);
    private SimpleUniverse u = null;
    String file;
    float boxSize;

    public BranchGroup createSceneGraph()
    {
		BranchGroup objRoot = new BranchGroup();
		TransformGroup objTrans = new TransformGroup();
		objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		objRoot.addChild(objTrans);

		StreamTokenizer tokenizer;
		try
		{		
			BufferedReader iFile = new BufferedReader(new FileReader(file));
			tokenizer = new StreamTokenizer(iFile);
			tokenizer.parseNumbers();
			tokenizer.whitespaceChars('\t','\t');
			tokenizer.whitespaceChars('\n','\n');
			tokenizer.whitespaceChars(' ',' ');

			double x = 0, y = 0, z = 0;
			float r = 0, g = 0, b = 0;
			double temp;
			int i = 1;
			int token;

			token = tokenizer.nextToken();
			while(token != StreamTokenizer.TT_EOF)
			{
				temp = 0;
				if(token == StreamTokenizer.TT_NUMBER)
					temp = tokenizer.nval;
	
				if(i == 1) x = temp;
				else if( i == 2) y = temp;
				else if( i == 3) z = temp;
				else if( i == 4) r = (float)temp;
				else if( i == 5) g = (float)temp;
				else if( i == 0) b = (float)temp;
	
				if(i == 0)
				{
					Appearance appearance = new Appearance();
					Color3f objColor = new Color3f(r,g,b);
					appearance.setMaterial(new Material(black, black, objColor, white, 80.0f));
					appearance.setLineAttributes(new LineAttributes(2.0f, LineAttributes.PATTERN_SOLID, true));
		
					TransformGroup objTrans1 = new TransformGroup();
					Transform3D t = new Transform3D();
					t.set(new Vector3d(x*boxSize,y*boxSize,z*boxSize));
					objTrans1.setTransform(t);
		
/*					Node obj = new Box(boxSize/2, boxSize/2, boxSize/2,
										Box.GENERATE_NORMALS | Box.GENERATE_TEXTURE_COORDS, appearance);*/
					Node obj = new Sphere(boxSize/2, Box.GENERATE_NORMALS | Box.GENERATE_TEXTURE_COORDS, appearance);
					objTrans1.addChild(obj);
					objRoot.addChild(objTrans1);
				}
				i = (i + 1)%6;
				token = tokenizer.nextToken();
			}
		}
		catch(FileNotFoundException e)
		{
			System.err.println(e.toString());
			System.exit(1);
		}
		catch(IOException e)
		{
			System.err.println(e.toString());
			System.exit(1);
		}

		BoundingSphere bounds = new BoundingSphere(new Point3d(0.0, 0.0, 0.0), Float.MAX_VALUE);
		BranchGroup lightsGroup = new BranchGroup();
		// Set up the background
		Color3f bgColor = new Color3f(1.0f, 1.0f, 1.0f);
		Background bg = new Background(bgColor);
		bg.setApplicationBounds(bounds);
		lightsGroup.addChild(bg);

		// Set up the global lights
		Color3f lColor1 = new Color3f(0.7f, 0.7f, 0.7f);
		Vector3f lDir1  = new Vector3f(-1.0f, -1.0f, -1.0f);
		Color3f alColor = new Color3f(0.2f, 0.2f, 0.2f);

		AmbientLight aLgt = new AmbientLight(alColor);
		aLgt.setInfluencingBounds(bounds);
		DirectionalLight lgt1 = new DirectionalLight(lColor1, lDir1);
		lgt1.setInfluencingBounds(bounds);
		lightsGroup.addChild(aLgt);
		lightsGroup.addChild(lgt1);
		objRoot.addChild(lightsGroup);

		return objRoot;
    }

    public CellSimClient(String[] args) {
    	file = args[0];
    	boxSize = new Float(args[1]).floatValue();
    }

    public void init() {
		// These are the string arguments given to the VirtualInputDevice
        // constructor.  These are settable parameters.  Look in the 
        // VirtualInputDevice constructor for a complete list.
        String[] args = new String[10];
        args[0] = "printvalues";
        args[1] = "true";
        args[2] = "xaxisrotinit";
        args[3] = "1.0";
        args[4] = "yaxisrotinit";
        args[5] = "1.0";
        args[6] = "zaxisrotinit";
        args[7] = "1.0";
        args[8] = null;

        InputDevice device = new VirtualInputDevice( args );

        // now create the CellSimClient Canvas
	setLayout(new BorderLayout());
        GraphicsConfiguration config =
           SimpleUniverse.getPreferredConfiguration();

        Canvas3D c = new Canvas3D(config);
	add("Center", c);

		// Create a simple scene and attach it to the virtual universe
	BranchGroup scene = createSceneGraph();
	u = new SimpleUniverse(c);

        // The InputDevice must be initialized before registering it 
        // with the PhysicalEnvironment object.
        device.initialize();

	// Register the VirtualInputDevice with Java 3D
	u.getViewer().getPhysicalEnvironment().addInputDevice( device );

	TransformGroup viewTrans = 
                        u.getViewingPlatform().getViewPlatformTransform();
	SensorBehavior s = new SensorBehavior( viewTrans, device.getSensor(0) );
	s.setSchedulingBounds( new BoundingSphere
                       ( new Point3d(0.0,0.0,0.0), Float.MAX_VALUE ));
	scene.addChild( s );
	u.addBranchGraph(scene);
    }

    public void destroy(){
	u.removeAllLocales();
    }

    public static void main(String[] args) {
	new MainFrame(new CellSimClient(args), 600, 600);
    }
}
