/*
 *	@(#)CellSimClient.java 1.11 02/04/01 15:03:52
 *
 * Copyright (c) 1996-2002 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistribution in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * Neither the name of Sun Microsystems, Inc. or the names of
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
 * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
 * EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN
 * OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR
 * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE SOFTWARE,
 * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * You acknowledge that Software is not designed,licensed or intended
 * for use in the design, construction, operation or maintenance of
 * any nuclear facility.
 */

import java.io.*;
import java.applet.Applet;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.event.WindowAdapter;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class CellSimClient extends Applet
{
	public final Color3f black = new Color3f(0f,0f,0f);
	public final Color3f white = new Color3f(0f,0f,0f);

	TransformGroup objTrans;
    double angle = 0.0;
    Transform3D trans = new Transform3D();

	boolean readParameters = false;
	String inFile = null;
	float boxSize = 1f;
	boolean isSphere = true;
	double angleX = 0.0;
	double angleY = 0.0;
	double angleZ = 0.0;

    JButton rotateX = new JButton(" X ");
    JButton rotateY = new JButton(" Y ");
    JButton rotateZ = new JButton(" Z ");
    JButton resetButton = new JButton("Reset");

    private SimpleUniverse u = null;

    public BranchGroup createSceneGraph()
    {
		// Create the root of the branch graph
		BranchGroup objRoot = new BranchGroup();

		// Create the transform group node and initialize it to the
		// identity.  Enable the TRANSFORM_WRITE capability so that
		// our behavior code can modify it at runtime.  Add it to the
		// root of the subgraph.
		objTrans = new TransformGroup();
		objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		objRoot.addChild(objTrans);

		StreamTokenizer tokenizer;
		try
		{		
			BufferedReader iFile = new BufferedReader(new FileReader(inFile));
			tokenizer = new StreamTokenizer(iFile);
			tokenizer.parseNumbers();
			tokenizer.whitespaceChars('\t','\t');
			tokenizer.whitespaceChars('\n','\n');
			tokenizer.whitespaceChars(' ',' ');

			double x = 0, y = 0, z = 0;
			float r = 0, g = 0, b = 0;
			double temp;
			int i = 1;
			int token;

			token = tokenizer.nextToken();
			while(token != StreamTokenizer.TT_EOF)
			{
				temp = 0;
				if(token == StreamTokenizer.TT_NUMBER)
					temp = tokenizer.nval;
	
				if(i == 1) x = temp;
				else if( i == 2) y = temp;
				else if( i == 3) z = temp;
				else if( i == 4) r = (float)temp;
				else if( i == 5) g = (float)temp;
				else if( i == 0) b = (float)temp;
	
				if(i == 0)
				{
					Appearance appearance = new Appearance();
					Color3f objColor = new Color3f(r,g,b);
					appearance.setMaterial(new Material(black, black, objColor, white, 80.0f));
					appearance.setLineAttributes(new LineAttributes(2.0f, LineAttributes.PATTERN_SOLID, true));
		
					TransformGroup objTrans1 = new TransformGroup();
					Transform3D t = new Transform3D();
					t.set(new Vector3d(x*boxSize,y*boxSize,z*boxSize));
					objTrans1.setTransform(t);

					Node obj;
					if(isSphere)
						obj = new Sphere(boxSize/2,
										Sphere.GENERATE_NORMALS | Sphere.GENERATE_TEXTURE_COORDS, appearance);
					else
						obj = new Box(boxSize/2, boxSize/2, boxSize/2,
										Box.GENERATE_NORMALS | Box.GENERATE_TEXTURE_COORDS, appearance);
					objTrans1.addChild(obj);
					objTrans.addChild(objTrans1);
				}
				i = (i + 1)%6;
				token = tokenizer.nextToken();
			}
		}
		catch(FileNotFoundException e)
		{
			System.err.println(e.toString());
			System.exit(1);
		}
		catch(IOException e)
		{
			System.err.println(e.toString());
			System.exit(1);
		}

		// create the AWTInteractionBehavior	
		AWTInteractionBehavior awtBehavior = new AWTInteractionBehavior(objTrans, angleX, angleY, angleZ);
		rotateX.addActionListener(awtBehavior);
		rotateY.addActionListener(awtBehavior);
		rotateZ.addActionListener(awtBehavior);
		resetButton.addActionListener(awtBehavior);
		BoundingSphere bounds = new BoundingSphere(new Point3d(0.0, 0.0, 0.0), Float.MAX_VALUE);
		awtBehavior.setSchedulingBounds(bounds);
		objRoot.addChild(awtBehavior);

		// Set up the background
		BranchGroup lightsGroup = new BranchGroup();
		Color3f bgColor = new Color3f(1.0f, 1.0f, 1.0f);
		Background bg = new Background(bgColor);
		bg.setApplicationBounds(bounds);
		lightsGroup.addChild(bg);

		// Set up the global lights
		Color3f lColor1 = new Color3f(0.7f, 0.7f, 0.7f);
		Vector3f lDir1  = new Vector3f(-1.0f, -1.0f, -1.0f);
		Color3f alColor = new Color3f(0.2f, 0.2f, 0.2f);

		AmbientLight aLgt = new AmbientLight(alColor);
		aLgt.setInfluencingBounds(bounds);
		DirectionalLight lgt1 = new DirectionalLight(lColor1, lDir1);
		lgt1.setInfluencingBounds(bounds);
		lightsGroup.addChild(aLgt);
		lightsGroup.addChild(lgt1);
		objRoot.addChild(lightsGroup);

		return objRoot;
    }

    public CellSimClient(String args[])
    {
    	try
    	{
    		inFile = args[0];
			boxSize = (float)Math.toRadians(new Double(args[1]).doubleValue());
			angleX = Math.toRadians(new Double(args[2]).doubleValue());
			angleY = Math.toRadians(new Double(args[3]).doubleValue());
			angleZ = Math.toRadians(new Double(args[4]).doubleValue());
			if(args[5].equals("box"))
				isSphere = false;
			else
				isSphere = true;
    	}
    	catch(ArrayIndexOutOfBoundsException e)
    	{
    		if(inFile == null)
    		{
    			System.err.println("");
    			System.exit(1);
    		}
    	}
    	readParameters = true;
    }

    public CellSimClient()
    {
    	readParameters = false;
    }

    public void init()
    {
    	if(!readParameters)
    	{
			String s;

			inFile = getParameter("inputFile");
			if (inFile != null)
				readParameters = true;

			s = getParameter("boxSize");
			if(s != null)
				boxSize = (float)Math.toRadians(new Double(s).doubleValue());
			s = getParameter("rotationX");
			if(s != null)
				angleX = (float)Math.toRadians(new Double(s).doubleValue());
			s = getParameter("rotationY");
			if(s != null)
				angleY = (float)Math.toRadians(new Double(s).doubleValue());
			s = getParameter("rotationZ");
			if(s != null)
				angleZ = (float)Math.toRadians(new Double(s).doubleValue());
			s = getParameter("boxSize");
			if(s != null)
			{
				if(s.equals("box"))
					isSphere = false;
				else
					isSphere = true;
			}
    	}
    	
		if(readParameters)
		{
			setLayout(new BorderLayout());
        	GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();
		
	        Canvas3D c = new Canvas3D(config);
			add("Center", c);

			JPanel p = new JPanel();
			p.add(rotateX);
			p.add(rotateY);
			p.add(rotateZ);
			p.add(resetButton);
			add("North", p);

			// Create a simple scene and attach it to the virtual universe
			BranchGroup scene = createSceneGraph();
			scene.setCapability( BranchGroup.ALLOW_BOUNDS_READ );
			u = new SimpleUniverse(c);
		
	        // This will move the ViewPlatform back a bit so the
	        // objects in the scene can be viewed.
	        u.getViewingPlatform().setNominalViewingTransform();

			u.addBranchGraph(scene);
		}
		else
		{
			setLayout(new BorderLayout());
			JLabel l = new JLabel("Error");
			add("Center", l);
		}
    }

    public void destroy()
    {
		u.removeAllLocales();
    }
    
    // The following allows HelloUniverse to be run as an application
    // as well as an applet
    public static void main(String[] args)
    {
		new MainFrame(new CellSimClient(args), 800, 800);
    }
}
